#ifndef CONVERTER_H
#define CONVERTER_H

#include <QCoreApplication>
#include <QDebug>
#include <QDir>
#include <QFileInfo>
#include <QString>
#include <QMap>

using namespace std;

/*! This is the class which converts different file formats. */
class Converter
{
public:
    Converter();
    void init();
    int convert_PDBtoXYZRN(QString &filename, QVector<float>* temperatures, QVector<QString>* bindingSites, QVector<QString>* atomNames, QVector<QString>* resNames, QVector<int>* aminoAcid, QMap<QString, int>* hydrophobicity, QVector<QVector<int>>* connectivity);

  private:
    QMap<QString, int> _hydrophobicity;

    class AtomRadii{
      public:
        /*! This is the struct which handles Atom radii. */
        struct AtomRadius{
            int n;
            float dist, radiusExplicit, radiusUnited;
            AtomRadius(int n, float dist, float radiusExplicit, float radiusUnited) : n(n), dist(dist), radiusExplicit(radiusExplicit), radiusUnited(radiusUnited){}
        };

        void addRadius(int n, float dist, float radiusExplicit, float radiusUnited);
        bool containsAtomID(int n);
        AtomRadius getAtomRadius(int n);

      private:
        std::vector<AtomRadius> radii;
      };

    AtomRadii* radii;

    class Atoms{
      public:
        /*! This is the struct which handles Atoms. */
        struct Atom{
            QString respat, atmpat;
            int atmnum;
            AtomRadii::AtomRadius radius;
            Atom(QString respat, QString atmpat, int atmnum, AtomRadii::AtomRadius radius) : respat(respat), atmpat(atmpat), atmnum(atmnum), radius(radius){}
        };

        bool compareAtomEntries(QString nameA, QString nameB);
        void addAtom(QString respat, QString atmpat, int atmnum, AtomRadii::AtomRadius radius);
        Atom getAtom(QString aname, QString resname);
        int getAtomCount();

      private:
        std::vector<Atom> atoms;
    };

    Atoms* atoms;

};

#endif // CONVERTER_H
