#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QFileDialog>
#include <QDebug>
#include <QFileInfo>
#include <QProcess>
#include <QProgressDialog>
#include <QProgressBar>
#include <QMessageBox>
#include <QDialogButtonBox>
#include <QRadioButton>
#include <QPushButton>
#include <QLabel>

#include <converter.h>
#include <colorselector.h>

namespace Ui {
  class MainWindow;
}

/*! This is the class which is the main window. */
class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  explicit MainWindow(QWidget *parent = 0);
  ~MainWindow();

signals:
    void loadMesh(QString path, QVector<float>* temperatures, QVector<QString>* bindingSites, QVector<QString>* atomNames, QVector<QString>* resNames, QVector<int>* aminoAcid, QMap<QString, int>* hydrophobicity, QVector<QVector<int>>* connectivity);
    void combinedRotationToggle(bool toggle);

protected:
    void closeEvent(QCloseEvent *event);

private slots:
  void on_actionLoad_File_triggered();
  void on_actionColor_Scheme_triggered();
  void on_actionCombined_Rotation_triggered();
  void updateProgress(int progress);
  void toggleUI(bool toggle);

private:
  Ui::MainWindow *ui;
  Converter *converter;
  ColorSelector* colorSelector;
  QProgressBar* progressBar;
  bool meshLoaded = false;
  bool combinedRotation = true;

};

#endif // MAINWINDOW_H
