#ifndef MESHSPHEREVIEW_H
#define MESHSPHEREVIEW_H

#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLExtraFunctions>
#include <QOpenGLFramebufferObject>
#include <QOpenGLTexture>
#include <QOpenGLFunctions>
#include <QOpenGLWidget>
#include <QQuaternion>
#include <QVector2D>
#include <QOpenGLBuffer>
#include <QFile>
#include <QWheelEvent>
#include <QKeyEvent>
#include <QVector3D>
#include <QtConcurrent/QtConcurrent>

#include "surfacemeshsphere.h"
#include "../meshview/surfacemesh.h"

/*! This is the class which powers the view that shows the sphere which is calculated from the mesh. */

class meshSphereView : public QOpenGLWidget, protected QOpenGLFunctions
{
  Q_OBJECT
public:
  meshSphereView(QWidget* parent = nullptr);

signals:
  void modelMatrixChangedMeshSphere(QMatrix4x4 modelMatrix); //!< Called when object rotation changes.
  void renderModeChangedMeshSphere(int renderMode); //!< Called when render mode changes.
  void surfaceMeshSphereChanged(SurfaceMeshSphere *newSurfaceMeshSphere); //!< Called when mesh changes.
  void toggleUI(bool toggle);

protected:
  void initializeGL() override;
  void paintGL() override;
  void resizeGL(int w, int h) override;

  void wheelEvent(QWheelEvent* e) override;
  void mousePressEvent(QMouseEvent *event) override;
  void mouseMoveEvent(QMouseEvent *event) override;
  void keyPressEvent(QKeyEvent *ke);

private slots:
  void newMeshLoaded(SurfaceMesh* mesh); //!< Called when new model is loaded.
  void sphereGenerated();
  void modelMatrixChangedMesh(QMatrix4x4 modelMatrix); //!< Called when object rotation changes.
  void renderModeChangedMesh(int renderMode); //!< Called when render mode is changed.
  void colorChanged(QVector<QVector3D>* colors); //!< Called when color is changed.
  void combinedRotationToggle(bool toggle); //!< Turn on/off synchronised cameras.

private:
  QOpenGLExtraFunctions *_gl;
  QOpenGLShaderProgram _m_program;

  QMatrix4x4 _modelMatrix, _viewMatrix, _projectionMatrix;
  QVector3D _cameraPosition, _cameraTarget;
  SurfaceMeshSphere* _currentMesh;

  int _renderMode = 1;
  bool _combinedRotation = true;
  QPoint _lastPos;
};

#endif // MESHSPHEREVIEW_H
