#ifndef SURFACEMESHSPHERE_H
#define SURFACEMESHSPHERE_H

#include <QOpenGLShaderProgram>
#include <QOpenGLExtraFunctions>
#include <QOpenGLFramebufferObject>
#include <QOpenGLTexture>
#include <QOpenGLBuffer>
#include <QVector3D>
#include <QMatrix4x4>
#include <QtMath>
#include <QOpenGLWidget>



/*! This class holds all the data for the transformed mesh. */

class SurfaceMeshSphere : public QOpenGLWidget {
    Q_OBJECT
    public:
        SurfaceMeshSphere();
        ~SurfaceMeshSphere();
        void initGL(QOpenGLExtraFunctions *gl);
        void set(QVector<QVector3D>* vertices, QVector<QVector3D>* normals, QVector<QVector3D>* color, QVector<GLuint>* indices, QVector3D center, float radius, QVector3D barycenter);
        void convertToSphere();
        void updateVerticesAndNormals();
        void draw();
        bool isLoaded();
        void changeColor(QVector<QVector3D> colors);

        QVector<QVector3D> *getVerticesSphere();
        QVector<QVector3D> *getNormalsSphere();

        QVector<QVector3D> *getColors();
        QVector<QVector2D> *getUVs();
        QVector<QVector2D> *getLongiLatis();
        QVector<GLuint> *getIndices();
        QVector3D getCenter() const;
        float getRadius() const;
        QVector3D getBarycenter() const;
signals:
        void updateProgress(int progress);

private:
        QOpenGLExtraFunctions *_gl;

        QVector<QVector3D> _verticesSphere;
        QVector<QVector3D> _normalsSphere;

        QVector<QVector3D> _vertices;
        QVector<QVector2D> _uvs;
        QVector<QVector2D> _longiLati;
        QVector<QVector3D> _normals;
        QVector<QVector3D> _colors;
        QVector<GLuint> _indices;
        GLuint _vao, _vbo, _ebo;
        QOpenGLBuffer _vertexBuffer, _normalBuffer, _colorBuffer, _indexBuffer;
        QVector3D _center, _barycenter;
        float _radius;
        bool loaded = false;

        struct Edge {
            int v1, v2;
        };
};

#endif // SURFACEMESHSPHERE_H
