#ifndef COLORCREATOR_H
#define COLORCREATOR_H

/*! Returns the default color for the current value. */
QVector3D getDefaultColor(float value){
    float r,g,b;
    float temp;
    if(value <= 1.0/8.0){
        temp = (value - 0.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 2.0 + temp * 140.0;
        g = (1.0 - temp) * 58.0 + temp * 44.0;
        b = (1.0 - temp) * 1.0 + temp * 0.0;
    }else if(value <= 2.0/8.0){
        temp = (value - 1.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 140.0 + temp * 145.0;
        g = (1.0 - temp) * 44.0 + temp * 224.0;
        b = (1.0 - temp) * 0.0 + temp * 0.0;
    }else if(value <= 3.0/8.0){
        temp = (value - 2.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 145.0 + temp * 0.0;
        g = (1.0 - temp) * 224.0 + temp * 250.0;
        b = (1.0 - temp) * 0.0 + temp * 255.0;
    }else if(value <= 4.0/8.0){
        temp = (value - 3.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 0.0 + temp * 174.0;
        g = (1.0 - temp) * 250.0 + temp * 0.0;
        b = (1.0 - temp) * 255.0 + temp * 255.0;
    }else if(value <= 5.0/8.0){
        temp = (value - 4.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 174.0 + temp * 255.0;
        g = (1.0 - temp) * 0.0 + temp * 124.0;
        b = (1.0 - temp) * 255.0 + temp * 53.0;
    }else if(value <= 6.0/8.0){
        temp = (value - 5.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 255.0 + temp * 135.0;
        g = (1.0 - temp) * 124.0 + temp * 255.0;
        b = (1.0 - temp) * 53.0 + temp * 137.0;
    }else if(value <= 7.0/8.0){
        temp = (value - 6.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 135.0 + temp * 221.0;
        g = (1.0 - temp) * 255.0 + temp * 232.0;
        b = (1.0 - temp) * 137.0 + temp * 255.0;
    }else if(value <= 8.0/8.0){
        temp = (value - 7.0/8.0)/(1.0/8.0);
        r = (1.0 - temp) * 221.0 + temp * 2.0;
        g = (1.0 - temp) * 232.0 + temp * 58.0;
        b = (1.0 - temp) * 255.0 + temp * 1.0;
    }
    r = r/255.0;
    g = g/255.0;
    b = b/255.0;
    return QVector3D(r,g,b);
}

/*! Returns the grey value for the current value. */
QVector3D getGreyValue(float value){
    float a, temp;

    if(value <= 1.0/2.0){
        temp = (value - 0.0/2.0)/(1.0/2.0);
        a = (1.0 - temp) * 191.0 + temp * 127.0;
    }else if(value <= 2.0/2.0){
        temp = (value - 1.0/2.0)/(1.0/2.0);
        a = (1.0 - temp) * 127.0 + temp * 63.0;
    }
    a = a/255.0;
    return QVector3D(a,a,a);
}

/*! Returns the color for the index 0. (Atoms) */
void SurfaceMesh::getColors0(QVector<QVector3D>* colors){
    int max = _atomNames->size();

    for(int i = 0; i < _vertices->size(); i++){
        float value = _atomNames->indexOf(_vertexNames->at(i).mid(0, _vertexNames->at(i).indexOf("_"))) / (float)max;
        colors->push_back(getDefaultColor(value));
    }
}

/*! Returns the color for the index 1. (Amino Acids) */
void SurfaceMesh::getColors1(QVector<QVector3D>* colors){
    int max = 0;
    for(int i = 0; i < _aminoAcid->size(); i++){
        if(_aminoAcid->at(i) > max)
            max = _aminoAcid->at(i);
    }

    for(int i = 0; i < _vertices->size(); i++){
        float value = 0;
        if(max != 0)
            value = _aminoAcid->at(_vertexClosestSpheres->at(i)-1) / (float)max;
        colors->push_back(getDefaultColor(value));
    }
}

/*! Returns the color for the index 2. (Residues) */
void SurfaceMesh::getColors2(QVector<QVector3D>* colors){
    int max = _resNames->size();

    for(int i = 0; i < _vertices->size(); i++){
        float value = _resNames->indexOf(_vertexNames->at(i).mid(_vertexNames->at(i).indexOf("_")+1, 3)) / (float)max;
        colors->push_back(getDefaultColor(value));
    }
}

/*! Returns the color for the index 3. (Elevation) */
void SurfaceMesh::getColors3(QVector<QVector3D>* colors){
    QVector3D c = getBarycenter();
    QVector<float> dist;
    float dMax = 0;
    for(int i = 0; i < _vertices->size(); i++){
        float d = _vertices->at(i).distanceToPoint(c);
        dist.push_back(d);
        if(d > dMax) dMax = d;
    }

    for(int i = 0; i < _vertices->size(); i++){
        float value = dist[i]/dMax;
        float r,g,b;
        float temp;

        if(value <= 1.0/3.0){
            temp = (value - 0.0/3.0)/(1.0/3.0);
            r = (1.0 - temp) * 50.0 + temp * 199.0;
            g = (1.0 - temp) * 131.0 + temp * 217.0;
            b = (1.0 - temp) * 89.0 + temp * 54.0;
        }else if(value <= 2.0/3.0){
            temp = (value - 1.0/3.0)/(1.0/3.0);
            r = (1.0 - temp) * 199.0 + temp * 132.0;
            g = (1.0 - temp) * 217.0 + temp * 54.0;
            b = (1.0 - temp) * 54.0 + temp * 49.0;
        }else if(value <= 3.0/3.0){
            temp = (value - 2.0/3.0)/(1.0/3.0);
            r = (1.0 - temp) * 132.0 + temp * 255.0;
            g = (1.0 - temp) * 54.0 + temp * 255.0;
            b = (1.0 - temp) * 49.0 + temp * 255.0;
        }
        r = r/255.0;
        g = g/255.0;
        b = b/255.0;
        colors->push_back(QVector3D(r, g, b));
    }
}

/*! Returns the color for the index 4. (Distance Surface Globe) */
void SurfaceMesh::getColors4(QVector<QVector3D>* colors){
    QVector3D c = getCenter();
    float r = getRadius();
    QVector<float> dist;
    for(int i = 0; i < _vertices->size(); i++){
        float d = abs(_vertices->at(i).distanceToPoint(c) - r);
        dist.push_back(d);
    }

    for(int i = 0; i < _vertices->size(); i++){
        float value = dist[i]/r;
        float r,g,b;
        float temp;

        if(value <= 1.0/2.0){
            temp = (value - 0.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 50.0 + temp * 199.0;
            g = (1.0 - temp) * 131.0 + temp * 217.0;
            b = (1.0 - temp) * 89.0 + temp * 54.0;
        }else if(value <= 2.0/2.0){
            temp = (value - 1.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 199.0 + temp * 132.0;
            g = (1.0 - temp) * 217.0 + temp * 54.0;
            b = (1.0 - temp) * 54.0 + temp * 49.0;
        }
        r = r/255.0;
        g = g/255.0;
        b = b/255.0;
        colors->push_back(QVector3D(r, g, b));
    }
}

/*! Returns the color for the index 6. (Temperature) */
void SurfaceMesh::getColors6(QVector<QVector3D>* colors){
    float min = 0.0, max = 100.0;
    for(int i = 0; i < _temperatures->size(); i++){
        if(_temperatures->at(i) < min)
            min = _temperatures->at(i);
        if(_temperatures->at(i) > max)
            max = _temperatures->at(i);
    }

    QVector3D c = getCenter();
    float r = getRadius();
    QVector<float> dist;
    for(int i = 0; i < _vertices->size(); i++){
        float d = abs(_vertices->at(i).distanceToPoint(c) - r);
        dist.push_back(d);
    }

    for(int i = 0; i < _vertices->size(); i++){
        float value = -1.0;
        value = (_temperatures->at(_vertexClosestSpheres->at(i)-1) - min) / (max - min);
        if(value == -1.0)
            value = 50.0;
        float r,g,b;
        float temp;

        if(value <= 1.0/2.0){
            temp = (value - 0.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 33.0 + temp * 235.0;
            g = (1.0 - temp) * 106.0 + temp * 235.0;
            b = (1.0 - temp) * 157.0 + temp * 235.0;
        }else if(value <= 2.0/2.0){
            temp = (value - 1.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 235.0 + temp * 172.0;
            g = (1.0 - temp) * 235.0 + temp * 67.0;
            b = (1.0 - temp) * 235.0 + temp * 61.0;
        }
        r = r/255.0;
        g = g/255.0;
        b = b/255.0;
        colors->push_back(QVector3D(r, g, b));
    }
}

/*! Returns the color for the index 7. (Hydrophobicity) */
void SurfaceMesh::getColors7(QVector<QVector3D>* colors){
    for(int i = 0; i < _vertices->size(); i++){
        float value = 0.0;
        QString res = _vertexNames->at(i).mid(_vertexNames->at(i).indexOf("_")+1, 3);

        if(_hydrophobicity->contains(res))
            value = (float)_hydrophobicity->find(res).value();

        value = (value - (-100.0)) / ((100.0) - (-100.0));
        float r,g,b;
        float temp;

        if(value <= 1.0/2.0){
            temp = (value - 0.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 33.0 + temp * 235.0;
            g = (1.0 - temp) * 106.0 + temp * 235.0;
            b = (1.0 - temp) * 157.0 + temp * 235.0;
        }else if(value <= 2.0/2.0){
            temp = (value - 1.0/2.0)/(1.0/2.0);
            r = (1.0 - temp) * 235.0 + temp * 172.0;
            g = (1.0 - temp) * 235.0 + temp * 67.0;
            b = (1.0 - temp) * 235.0 + temp * 61.0;
        }
        r = r/255.0;
        g = g/255.0;
        b = b/255.0;
        colors->push_back(QVector3D(r, g, b));
    }
}

/*! Returns the color for the index 9. (Binding Sites) */
void SurfaceMesh::getColors9(QVector<QVector3D>* colors){
    QVector3D c = getCenter();
    float r = getRadius();
    QVector<float> dist;
    for(int i = 0; i < _vertices->size(); i++){
        float d = abs(_vertices->at(i).distanceToPoint(c) - r);
        dist.push_back(d);
    }

    for(int i = 0; i < _vertices->size(); i++){
        if(_bindingSites->contains(_vertexNames->at(i))){
            float value = _bindingSites->indexOf(_vertexNames->at(i)) / (float)_bindingSites->size();
            colors->push_back(getDefaultColor(value));
        }else{
            float value = dist[i]/r;
            colors->push_back(getGreyValue(value));
        }
    }
}

/*! Returns the color for the index 10. (Connectivity) */
void SurfaceMesh::getColors10(QVector<QVector3D>* colors){
    QVector3D c = getCenter();
    float r = getRadius();
    QVector<float> dist;
    for(int i = 0; i < _vertices->size(); i++){
        float d = abs(_vertices->at(i).distanceToPoint(c) - r);
        dist.push_back(d);
    }

    QVector<int> con;
    for(int i = 0; i < _connectivity->size(); i++){
        for(int j = 0; j < _connectivity->at(i).size(); j++){
            con.push_back(_connectivity->at(i).at(j));
        }
    }

    for(int i = 0; i < _vertexClosestSpheres->size(); i++){
        int count = con.count(_vertexClosestSpheres->at(i)-1);
        if(count > 0){
            QVector3D c = QVector3D(0,0,0);
            for(int j = 0; j < _connectivity->size(); j++){
                if(_connectivity->at(j).contains(_vertexClosestSpheres->at(i)-1)){
                    float value = j / (float)_connectivity->size();
                    c += getDefaultColor(value);
                }

            }
            colors->push_back(c/(float)count);
        }else{
            float value = dist[i]/r;
            colors->push_back(getGreyValue(value));
        }
    }
}

/*! Returns the color for the given index. */
void SurfaceMesh::getColors(QVector<QVector3D>* colors, int color){
    colors->clear();

    switch(color){
        case 0:
            getColors0(colors);
            break;
        case 1:
            getColors1(colors);
            break;
        case 2:
            getColors2(colors);
            break;
        case 3:
            getColors3(colors);
            break;
        case 4:
            getColors4(colors);
            break;
        case 5:
            //getColors5(colors);
            //break;
        case 6:
            getColors6(colors);
            break;
        case 7:
            getColors7(colors);
            break;
        case 8:
            //getColors8(colors);
            //break;
        case 9:
            getColors9(colors);
            break;
        case 10:
            getColors10(colors);
            break;
        default:
            getColors0(colors);
            break;
    }
}



#endif // COLORCREATOR_H
