#ifndef MESHVIEW_H
#define MESHVIEW_H

#include <QMatrix4x4>
#include <QOpenGLShaderProgram>
#include <QOpenGLExtraFunctions>
#include <QOpenGLFramebufferObject>
#include <QOpenGLFunctions>
#include <QOpenGLWidget>
#include <QQuaternion>
#include <QVector2D>
#include <QOpenGLBuffer>
#include <QFile>
#include <QWheelEvent>
#include <QKeyEvent>
#include <QVector3D>
#include <QMessageBox>

#include "surfacemesh.h"

/*! This is the class which powers the view that shows the raw mesh. */

class meshView : public QOpenGLWidget, protected QOpenGLFunctions
{
  Q_OBJECT
public:
  meshView(QWidget* parent = nullptr);

signals:
  void newMeshLoaded(SurfaceMesh* mesh); //!< Called when new model is loaded.
  void modelMatrixChangedMesh(QMatrix4x4 modelMatrix); //!< Called when object rotation changes.
  void renderModeChangedMesh(int renderMode); //!< Called when render mode changes.
  void colorChanged(QVector<QVector3D>* colors); //!< Called when colors changed.

protected:
  void initializeGL() override;
  void paintGL() override;
  void resizeGL(int w, int h) override;

  void wheelEvent(QWheelEvent* e) override;
  void mousePressEvent(QMouseEvent *event) override;
  void mouseMoveEvent(QMouseEvent *event) override;
  void keyPressEvent(QKeyEvent *ke);

private slots:
  void loadMesh(QString path, QVector<float>* temperatures, QVector<QString>* bindingSites, QVector<QString>* atomNames, QVector<QString>* resNames, QVector<int>* aminoAcid, QMap<QString, int>* hydrophobicity, QVector<QVector<int>>* connectivity);
  void changeColor(int colorSelection); //!< Called when colors changed.
  void modelMatrixChangedMeshSphere(QMatrix4x4 modelMatrix); //!< Called when object rotation changes.
  void renderModeChangedMeshSphere(int renderMode); //!< Called when render mode is changed.
  void combinedRotationToggle(bool toggle); //!< Turn on/off synchronised cameras.

private:
  QOpenGLExtraFunctions *_gl;
  QOpenGLShaderProgram _m_program;
  SurfaceMesh* _currentMesh;

  QMatrix4x4 _modelMatrix, _viewMatrix, _projectionMatrix;
  QVector3D _cameraPosition, _cameraTarget;

  int _renderMode = 1;
  bool _combinedRotation = true;
  QPoint _lastPos;
};

#endif // MESHVIEW_H
