#ifndef SURFACEMESH_H
#define SURFACEMESH_H

#include <QOpenGLShaderProgram>
#include <QOpenGLExtraFunctions>
#include <QOpenGLBuffer>
#include <QVector3D>
#include <QMatrix4x4>

/*! Holds the data about the loaded mesh. */

class SurfaceMesh {
    public:
        SurfaceMesh(QOpenGLExtraFunctions *gl);
        ~SurfaceMesh();
        void set(QVector<QVector3D>* vertices, QVector<QVector3D>* normals, QVector<GLuint>* vertexFaceNumbers, QVector<GLuint>* vertexClosestSpheres, QVector<GLuint>* vertexTypes, QVector<QString>* vertexNames, QVector<GLuint>* indices, QVector<GLuint>* faceTypes, QVector<GLuint>* faceNumbers, QVector<float>* temperatures, QVector<QString>* bindingSites, QVector<QString>* atomNames, QVector<QString>* resNames, QVector<int>* aminoAcid, QMap<QString, int>* hydrophobicity, QVector<QVector<int>>* connectivity, QVector3D center, float radius, QVector3D barycenter);
        void getColors(QVector<QVector3D>* colors, int color);
        void getColors0(QVector<QVector3D>* colors);
        void getColors1(QVector<QVector3D>* colors);
        void getColors2(QVector<QVector3D>* colors);
        void getColors3(QVector<QVector3D>* colors);
        void getColors4(QVector<QVector3D>* colors);
        void getColors6(QVector<QVector3D>* colors);
        void getColors7(QVector<QVector3D>* colors);
        void getColors9(QVector<QVector3D>* colors);
        void getColors10(QVector<QVector3D>* colors);

        void draw();
        bool isLoaded();
        void changeColor(int colorSelection);

        QVector<QVector3D> *getVertices() const;
        QVector<QVector3D> *getNormals() const;
        QVector<QVector3D> *getColors() const;
        QVector<GLuint> *getIndices() const;
        QVector3D getCenter() const;
        float getRadius() const;
        QVector3D getBarycenter() const;

private:
        QOpenGLExtraFunctions *_gl;
        QVector<QVector3D> *_vertices; 
        QVector<QVector3D> *_normals;
        QVector<QVector3D> *_colors;
        QVector<GLuint> *_indices;
        QVector<GLuint> *_vertexFaceNumbers, *_vertexClosestSpheres, *_vertexTypes;
        QVector<QString> *_vertexNames;
        QVector<GLuint> *_faceTypes, *_faceNumbers;
        QVector<float> *_temperatures;
        QVector<QString> *_bindingSites;
        QVector<int> *_aminoAcid;
        QMap<QString, int>* _hydrophobicity;
        QVector<QString> *_atomNames, *_resNames;
        QVector<QVector<int>>* _connectivity;
        GLuint _currentColor = 0;
        GLuint _vao, _vbo, _ebo;
        QOpenGLBuffer _vertexBuffer, _normalBuffer, _colorBuffer, _indexBuffer;
        QVector3D _center, _barycenter;
        float _radius;
        bool loaded = false;
};

#endif // SURFACEMESH_H
