#include "textureview.h"

textureView::textureView(QWidget *parent) : QOpenGLWidget(parent)
{
    QSurfaceFormat f = QSurfaceFormat::defaultFormat();
    QSysInfo sysinfo;
    if(sysinfo.productType().toCaseFolded() == QString("osx") ||
     sysinfo.productType().toCaseFolded() == QString("macos")){
        f.setVersion( 4, 1 );
        f.setProfile(QSurfaceFormat::CoreProfile);
    } else {
        f.setVersion( 4, 3 );
    }
    setFormat(f);
    QSurfaceFormat::setDefaultFormat(f);
}

void textureView::initializeGL()
{
    initializeOpenGLFunctions();
    _gl = this->context()->extraFunctions();
    sm.init(_gl);

    sphereMapShader = new shader(":/shaders/sphereToMap.vert", ":/shaders/sphereToMap.geom", ":/shaders/sphereToMap.frag");
    sphereMapShader->getShaderProgramm()->bindAttributeLocation("vertexPosition", 0);
    sphereMapShader->getShaderProgramm()->bindAttributeLocation("vertexNormal", 1);
    sphereMapShader->getShaderProgramm()->bindAttributeLocation("vertexColor", 2);

    glClearColor(0.8, 0.8, 0.8, 1);
    glEnable(GL_DEPTH_TEST);
}

void textureView::paintGL()
{
    if(_sphere != NULL && _sphere->isLoaded()){
        sm.setSphere(_sphere);

        sphereMapShader->getShaderProgramm()->bind();
        sm.draw();
        sphereMapShader->getShaderProgramm()->release();
    }
}

void textureView::surfaceMeshSphereChanged(SurfaceMeshSphere *newSurfaceMeshSphere)
{
    _sphere = newSurfaceMeshSphere;
    this->update();
}


